/*
* TFT module must be initialized.
* Soft_UART_Helper.h must be included.
*/

#include "SF_driver.h"
#include "Serial_Flash_Util.h"
#include "Soft_UART_Helper.h"

#define BUFFER_GEN_LEN 64
#define BUFFER_SERIAL_LEN 512

static unsigned char g_buffer[BUFFER_GEN_LEN] = { 0 };
static unsigned char serial_buffer[BUFFER_SERIAL_LEN] = { 0 };
static unsigned int g_row = 0;
static unsigned int g_col = 5;
static unsigned int g_incr = 15;

// this is for tft text ouput
void SerialFlashUtilInit()
{
  g_row = 5; // y axis
  g_col = 5; // x axis
  // any other init code here
}

// copy bmp from resource to serial flash
// note: to erase chip first, supply 1 to erase_chip argument
// image: .bmp
// serial flash starting addr: 0x0000
unsigned long CopyImageToSerialFlash(const code char *image, unsigned int image_length, char erase_chip)
{
  unsigned long flash_addr = 0x0000; // start address of flash memory
  unsigned long image_flash_address = flash_addr;
  unsigned int i = 0;
  char* image_ptr = image;

  if (image == 0) { return 0; }

  if (erase_chip == 1)
  {
    TFT_Write_Text("Erasing chip... ", g_col, g_row+=g_incr);
    SerialFlash_ChipErase();
    TFT_Write_Text("Done", g_col, g_row+=g_incr);
  }

  TFT_Write_Text("Copying image to serial flash...", g_col, g_row+=g_incr);
  for (i = 0; i < image_length; i++)
  {
    //ByteToStr(addr, &c_1);
    // void SerialFlash_WriteByte(unsigned char _data, unsigned long address)
    SerialFlash_WriteByte(*image_ptr, flash_addr);
    image_ptr++;
    flash_addr++;
  }
  TFT_Write_Text("Done", g_col, g_row+=g_incr);

  // verify:
  TFT_Write_Text("Verifying...", g_col, g_row+=g_incr);
  image_ptr = image;
  flash_addr = 0x0000;
  for (i = 0; i < image_length; i++)
  {
    if (*image_ptr != SerialFlash_ReadByte(flash_addr))
    {
      TFT_Write_Text("ERROR", g_col, g_row+=g_incr);
      return 0;
    }
    image_ptr++;
    flash_addr++;
  }
  TFT_Write_Text("Done", g_col, g_row+=g_incr);

  return image_flash_address;
}

// [in] offset: start in memory from which to read
// [in] count:  how much bytes to read
// [out] nu:    how much bytes are read
char* FlashReadHandler(unsigned long offset, unsigned long count, unsigned long* num)
{
  const unsigned long flash_addr = 0x00000000; // beginning of serial flash memory
  /*unsigned int print_offset = 0;*/
  int i = 0;

  if (count > BUFFER_SERIAL_LEN) Soft_UART_WriteText("error: buffer overflow");

  // output
  /*memset(g_buffer, 0, BUFFER_LENGTH);
  print_offset = sprintf(g_buffer, "offset %7u;  ", offset);
  print_offset += sprintf(g_buffer + print_offset, "count %5u;  ", count);*/
  // output cont

  /*memset(serial_buffer, 0, BUFFER_LENGTH);
  for (i = 0; i < count; i++)
  {
    serial_buffer[i] = SerialFlash_ReadByte(flash_addr + offset + i); // racunam da se svaki put offset inkrementira
  }
  *num = i;*/

  memset(serial_buffer, 0, BUFFER_SERIAL_LEN);
  SerialFlash_ReadArray(offset, serial_buffer, count);
  *num = count;

  // output resume
  /*sprintf(g_buffer + print_offset, "*num %5u;  ", *num);
  Soft_UART_WriteText(g_buffer);*/
  // output end

  // return pointer to beginning of each row in flash memory
  return serial_buffer;
}
